/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.qrscanner;

import java.io.IOException;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.List;

import javax.jmdns.JmDNS;
import javax.jmdns.ServiceEvent;
import javax.jmdns.ServiceInfo;
import javax.jmdns.ServiceListener;

import com.github.sarxos.webcam.Webcam;
import com.github.sarxos.webcam.WebcamCompositeDriver;
import com.github.sarxos.webcam.WebcamDevice;
import com.github.sarxos.webcam.WebcamDiscoveryEvent;
import com.github.sarxos.webcam.WebcamDiscoveryListener;
import com.github.sarxos.webcam.ds.buildin.WebcamDefaultDriver;
import com.github.sarxos.webcam.ds.ipcam.IpCamDevice;
import com.github.sarxos.webcam.ds.ipcam.IpCamDeviceRegistry;
import com.github.sarxos.webcam.ds.ipcam.IpCamDriver;
import com.github.sarxos.webcam.ds.ipcam.IpCamMode;

import de.frankmuenster.jameica.qrscanner.ipcam.DroidCamDevice;
import de.frankmuenster.jameica.qrscanner.ipcam.IphoneIpCameraCamDevice;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.logging.Logger;

/**
 * Webcam berwachung implementiert das {@link com.github.sarxos.webcam.WebcamDiscoveryListener} Interface. Es ermittelt
 * die max. Auflsung einer neuen gefunden Webcam.
 *
 * @author Frank Mnster
 */
@SuppressWarnings("nls")
public class WebcamWatchdog implements WebcamDiscoveryListener, ServiceListener {

    /**
    *
    */
    public static final String NO_WEBCAM_FOUND = "Keine Webcams gefunden.";

    /**
     * Private Instanz
     */
    private static WebcamWatchdog instance;

    /**
     * Das Java mDns Objekt
     */
    private transient JmDNS jmdns;

    /**
     * Der Composite-Treiber fr die Webcams
     */
    private static MyCompositeDriver driver = new MyCompositeDriver();

    /**
     * Customised Webcam Treiber. wir untersttzen sowohl Standard(USB)- als auch IP-Webcams.
     */
    private static class MyCompositeDriver extends WebcamCompositeDriver {

        /**
         * Unser Composite Treiber
         */
        public MyCompositeDriver() {
            super();
            this.add(new WebcamDefaultDriver());
            this.add(new IpCamDriver());
        }
    }

    // register custom composite driver
    static {
        // Webcam.setHandleTermSignal(true);
        Webcam.setDriver(WebcamWatchdog.driver);
    }

    /**
     * Liefert den aktuellen Treiber.
     *
     * @return der Composite-Treiber.
     */
    public static MyCompositeDriver getDriver() {
        return WebcamWatchdog.driver;
    }

    /**
     * Liefert <i>true</i> falls die Webcam
     *
     * @return initialised
     */
    // public static Boolean isInitiaised(Webcam webcam) throws WebcamException
    // {
    // for (MyWebcam myWebCam : webcams) {
    // if (myWebCam.getWebcamName().equals(webcam.getName())) {
    // return myWebCam.initialised.get();
    // }
    // }
    // throw new WebcamException("Webcam nicht gefunden");
    // }
    /**
     * Registriert die Klasse im WebcamDiscoveryListener und initialisiert die Webcam Klasse.
     */
    private WebcamWatchdog() {
        WebcamWatchdog.getDefault();
        // Webcam.getDefault();
        // JmDNS initialisieren
        if (this.jmdns == null) {
            try {
                this.jmdns = JmDNS.create(InetAddress.getLocalHost());
                // Mini Webcam App(iphone)
                this.jmdns.addServiceListener("_mini_webcam_101._tcp.local.", this);
                this.jmdns.addServiceListener("_aircamera._tcp.local.", this);
            } catch (final UnknownHostException e) {
                Logger.error("Unbekannter Host angegeben.", e);
            } catch (final IOException e) {
                Logger.error("I/O-Fehler.", e);
            }
        }
        // IpCams
        final String camName = Settings.getIpCam();
        if (!Settings.NO_IP_CAM.equals(camName)) {
            try {
                IpCamDevice ipCam = null;
                switch (camName) {
                    case DroidCamDevice.CAM_NAME:
                        Logger.debug("Adding DroidCam App device: " + Settings.getIpCamHost() + ":"
                                + Settings.getIpCamPort());
                        ipCam = new DroidCamDevice(Settings.getIpCamHost(), Settings.getIpCamPort());
                        break;
                    case IphoneIpCameraCamDevice.CAM_NAME:
                        Logger.debug("Adding Iphone IP Camera App device: " + Settings.getIpCamHost() + ":"
                                + Settings.getIpCamPort());
                        ipCam = new IphoneIpCameraCamDevice(Settings.getIpCamHost(), Settings.getIpCamPort());
                        break;
                    default:
                        Logger.debug(
                                "Adding standard IP cam: " + Settings.getIpCamHost() + ":" + Settings.getIpCamPort());
                        final StringBuffer urlBuilder = new StringBuffer("http://").append(Settings.getIpCamHost())
                                .append(':').append(Settings.getIpCamPort());
                        ipCam = new IpCamDevice(Settings.STANDRD_IP_CAM, urlBuilder.toString(), IpCamMode.PUSH);
                        break;
                }
                if (ipCam != null) {
                    IpCamDeviceRegistry.register(ipCam);
                }
            } catch (final MalformedURLException e) {
                // Nur loggen
                Logger.error("Die Adresse der IP Kamera ist falsch.", e);
                GUI.getStatusBar().setErrorText("IpCam IP Adresse ist nicht korret formatiert.");
            }
        }
    }

    /**
     *
     */
    public static void getDefault() {
        // In eigenen >Thread, da es sonst am MAC hngend
        new Thread(new Runnable() {

            /**
             * @see java.lang.Runnable#run()
             */
            @Override
            public void run() {
                Webcam.getDefault();
            }
        }).start();
    }

    /**
     * Liefert die Instance des Watchdogs
     *
     * @return die WatchDog-Instanz
     */
    public static WebcamWatchdog getInstance() {
        synchronized (WebcamWatchdog.class) {
            if (WebcamWatchdog.instance == null) {
                WebcamWatchdog.instance = new WebcamWatchdog();
            }
        }
        return WebcamWatchdog.instance;
    }

    /**
     * @see com.github.sarxos.webcam.WebcamDiscoveryListener#webcamFound(com.github.sarxos.webcam.WebcamDiscoveryEvent)
     */
    @Override
    public void webcamFound(final WebcamDiscoveryEvent event) {
        // Logger.debug("Watchdog - Webcam has been connected: " +
        // event.getWebcam().getName());
        //
        // // add to list
        // MyWebcam webcam = new MyWebcam(event.getWebcam().getName(),
        // event.getWebcam().getDriver().getClass());
        // webcams.add(webcam);
        //
        // for (MyWebcam element : webcams) {
        // Logger.debug(" " + element.getWebcamName() + " Class: " +
        // element.getWebcamDriver());
        // }
    }

    /**
     * @see com.github.sarxos.webcam.WebcamDiscoveryListener#webcamGone(com.github.sarxos.webcam.WebcamDiscoveryEvent)
     */
    @Override
    public void webcamGone(final WebcamDiscoveryEvent event) {
        // Nothing to do here
        Logger.debug("Webcame removed: " + event.getWebcam().getName());
    }

    /**
     * @see javax.jmdns.ServiceListener#serviceAdded(javax.jmdns.ServiceEvent)
     */
    @Override
    public void serviceAdded(final ServiceEvent event) {
        // Nothing to do here
        Logger.debug("Service added: " + event.getInfo());
    }

    /**
     * @see javax.jmdns.ServiceListener#serviceRemoved(javax.jmdns.ServiceEvent)
     */
    @Override
    public void serviceRemoved(final ServiceEvent event) {
        final ServiceInfo serviceInfo = event.getInfo();
        Logger.debug("Watchdog - IpWebcam has been disconnected: " + serviceInfo.getName());
        final String webcamName = serviceInfo.getName();
        if (IpCamDeviceRegistry.isRegistered(webcamName)) {
            Logger.info("De-registiere Webcam " + webcamName);
            IpCamDeviceRegistry.unregister(webcamName);
        }
    }

    /**
     * @see javax.jmdns.ServiceListener#serviceResolved(javax.jmdns.ServiceEvent)
     */
    @Override
    public void serviceResolved(final ServiceEvent event) {
        Logger.debug("Service resolved: " + event.getInfo());
        Logger.debug("Service application: " + event.getInfo().getApplication());
        final ServiceInfo serviceInfo = event.getInfo();
        if (serviceInfo.getURLs().length > 0) {
            Logger.debug("Service url: " + event.getInfo().getURLs()[0]);
            try {
                final String webcamName = serviceInfo.getName();
                final String url = serviceInfo.getURLs()[0];
                final IpCamDevice ipCamDevice = new IpCamDevice(webcamName, url, IpCamMode.PUSH);
                if (!IpCamDeviceRegistry.isRegistered(ipCamDevice)) {
                    Logger.info("Registiere Webcam " + webcamName);
                    IpCamDeviceRegistry.register(ipCamDevice);
                }
            } catch (final MalformedURLException e) {
                Logger.error("Falsch formatierte URL.", e);
            }
        }
    }

    /**
     * Entfernt alle manuellen IP Kameras aus der Webcam-Registrierung
     */
    public static void unregisterIpCams() {
        if (IpCamDeviceRegistry.isRegistered(DroidCamDevice.CAM_NAME)) {
            // ipCam.dispose();
            Logger.debug("Successful Unregister DroidCam: " + IpCamDeviceRegistry.unregister(DroidCamDevice.CAM_NAME));
        }
        if (IpCamDeviceRegistry.isRegistered(IphoneIpCameraCamDevice.CAM_NAME)) {
            // ipCam.dispose();
            Logger.debug("Successful Unregister IphoneCam: "
                    + IpCamDeviceRegistry.unregister(IphoneIpCameraCamDevice.CAM_NAME));
        }
        if (IpCamDeviceRegistry.isRegistered(Settings.STANDRD_IP_CAM)) {
            // ipCam.dispose();
            Logger.debug("Successful Unregister IpCam: " + IpCamDeviceRegistry.unregister(Settings.STANDRD_IP_CAM));
        }
    }

    /**
     * Liefert eine Liste mit den Namen aller registrierten Webcams. Der Treiber wird neu geladen, um auch neu UpnD
     * Kameras zu listen.
     *
     * @return Liste mit den Namen aller registrierten Webcams
     */
    public List<String> getWebcamNames() {
        // Aktualisiere Webcamliste
        Webcam.resetDriver();
        Webcam.setDriver(WebcamWatchdog.getDriver());
        WebcamWatchdog.getDefault();
        // Webcam.getDefault();
        final List<String> webcams = new ArrayList<>();
        int camNumber = 1;
        for (final WebcamDevice webcam : Webcam.getDriver().getDevices()) {
            Logger.debug("Webcam (" + camNumber++ + "): " + webcam.getName());
            webcams.add(webcam.getName());
        }
        if (webcams.isEmpty()) {
            webcams.add(WebcamWatchdog.NO_WEBCAM_FOUND);
        }
        return webcams;
    }
}
