/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.qrscanner.gui.action;

import de.frankmuenster.jameica.qrcodehandler.QrCodeHandler;
import de.frankmuenster.jameica.qrcodehandler.QrCodeHandlerException;
import de.frankmuenster.jameica.qrcodehandler.QrCodeHandlerRegsistry;
import de.frankmuenster.jameica.qrscanner.gui.dialog.QrScannerDialog;
import de.frankmuenster.jameica.qrscanner.utils.I18N;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.system.Application;
import de.willuhn.jameica.system.OperationCanceledException;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * View und Kontrolleur zum Scannen und weiter bearbeiten eines QR-Codes.
 *
 * @author Frank Mnster
 */
public class QrScannerAction implements Action {

    /**
     * Standardkonstruktor
     */
    public QrScannerAction() {

        // Absichtlich leer
    }

    /**
     * ffnet einen QrScannerDialog und gibt das Ergebnis an einen passenden QR-Code Handler weiter, falls mglich.
     */
    @Override
    public void handleAction(final Object context) throws ApplicationException {

        String qrText = null;
        // show dialog and start scanning
        try {
            final QrScannerDialog dialog = new QrScannerDialog();
            try {
                qrText = dialog.open();
            } catch (final OperationCanceledException e) {
                Logger.info(I18N.tr("QrScannerAction.scanCancelled")); //$NON-NLS-1$
            } catch (final Exception e) { // NOPMD
                Logger.error(I18N.tr("QrScannerAction.errorOpenDialog"), e); //$NON-NLS-1$
            }
            if (qrText == null) {
                Logger.debug(I18N.tr("QrScannerAction.noQrCodeFound")); //$NON-NLS-1$
            } else {
                Logger.debug("Scann Result:\n" + qrText); //$NON-NLS-1$
                final QrCodeHandler handler = QrCodeHandlerRegsistry.getQrCodeHandler(qrText);
                if (handler == null) {
                    Logger.info(I18N.tr("QrScannerAction.noMatchingQrCodeHandler")); //$NON-NLS-1$
                    final String niceText = I18N.tr("QrScannerAction.noMatchingQrCodeHandlerMsg"); //$NON-NLS-1$

                    try {
                        Application.getCallback().notifyUser(niceText);
                    } catch (final Exception e) { // NOPMD
                        // Hier brauchen wir nichts zu tun!
                        Logger.error(I18N.tr("QrScannerAction.errorInfoDialog"), e); //$NON-NLS-1$
                    }
                } else {
                    try {
                        handler.handleQrCode(qrText);
                    } catch (final QrCodeHandlerException qre) {
                        Logger.error(I18N.tr("QrScannerAction.qrCodeError", qre.getMessage()), qre); //$NON-NLS-1$
                        final String niceText = I18N.tr("QrScannerAction.qrCodeReadError"); //$NON-NLS-1$
                        try {
                            Application.getCallback().notifyUser(niceText);
                        } catch (final Exception e) { // NOPMD
                            // Hier brauchen wir nichts zu tun!
                            Logger.error(I18N.tr("QrScannerAction.errorInfoDialog"), e); //$NON-NLS-1$
                        }
                    }
                }
            }
        } catch (final OperationCanceledException oce) {
            Logger.info(I18N.tr("QrScannerAction.scanCancelled")); //$NON-NLS-1$
            try {
                Thread.sleep(1000);
            } catch (final InterruptedException e) {
            }
        }
    }
}
