/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.qrscanner.gui.parts;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import com.github.sarxos.webcam.Webcam;
import com.github.sarxos.webcam.WebcamDevice;

import de.frankmuenster.jameica.qrscanner.WebcamWatchdog;
import de.frankmuenster.jameica.qrscanner.gui.dialog.threads.WebcamSelectedListener;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.input.SelectInput;
import de.willuhn.logging.Logger;

/**
 * Ein Webcam Dropdown Feld zur Auzswahl einer Webcam
 *
 * @author Frank Mnster
 */
public class WebcamSelector implements Part {

    /**
     * Anzeigetext, falls keine Webkamera gefunden wurde
     */
    private static final String NO_WEBCAM_FOUND = "Keine Webcam gefunden."; //$NON-NLS-1$

    /**
     * Die Default-Webcam
     */
    private transient WebcamDevice defaultWebcam;

    /**
     * Unsere Listeners fr Webcam-nderungen
     */
    private final transient Set<WebcamSelectedListener> listeners = new CopyOnWriteArraySet<>();

    /**
     * Das Dropdown-Select-Element mit alle Webcams
     */
    private transient SelectInput webcamList;

    /**
     * Konstruktor, der die Default Webcam ermittelt Standardkonstruktor
     */
    public WebcamSelector() {
        if (Webcam.getDefault() != null) {
            this.defaultWebcam = Webcam.getDefault().getDevice();
        }
    }

    /**
     * Fuegt eine Listener hinzu
     *
     * @param listener
     *            WebcamChangeListener der hinzugefgt werden soll
     */
    public final void addListener(final WebcamSelectedListener listener) {
        this.listeners.add(listener);
    }

    /**
     * Liefert den Namen der ausgewhlten Webcam oder NULL, falls keine Webcam gefunden wurde.
     *
     * @return den Namen der ausgewhlten Webcam oder NULL
     */
    public String getValue() {
        return (String) (this.webcamList.getValue() == null ? WebcamSelector.NO_WEBCAM_FOUND
                : this.webcamList.getValue());
    }

    /**
     * @see de.willuhn.jameica.gui.Part#paint(Composite)
     */
    @Override
    public void paint(final Composite parent) throws RemoteException {
        if (parent == null) {
            Logger.error("Parent is null"); //$NON-NLS-1$
        } else {
            final Composite container = new Composite(parent, SWT.NONE);
            final GridLayout gridLayout = new GridLayout();
            gridLayout.numColumns = 1;
            container.setLayout(gridLayout);
            if (this.webcamList == null) {
                // Gibt es berhaupt eine Webcam
                // final List<String> webcams = this.getWebcamNames();
                if (this.defaultWebcam == null) {
                    this.webcamList = new SelectInput(this.getWebcamNames(), null);
                    this.webcamList.setEnabled(false);
                } else {
                    // final String defaultWebcamName =
                    // this.defaultWebcam.getName();
                    this.webcamList = new SelectInput(this.getWebcamNames(), this.defaultWebcam.getName());
                    this.webcamList.paint(container);
                    this.webcamList.getControl().addListener(SWT.Selection, new Listener() {

                        /**
                         * @see org.eclipse.swt.widgets.Listener#handleEvent(Event)
                         */
                        @Override
                        public void handleEvent(final Event arg0) {
                            WebcamSelector.this.notifyListeners();
                        }
                    });
                }
            }
        }
    }

    /**
     * Entfernt einen Listener aus der Liste
     *
     * @param listener
     *            WebcamChangeListener der entfernt werden soll
     */
    public final void removeListener(final WebcamSelectedListener listener) {
        this.listeners.remove(listener);
    }

    private List<String> getWebcamNames() {
        // Aktualisiere Webcamliste
        Webcam.resetDriver();
        Webcam.setDriver(WebcamWatchdog.getDriver());
        Webcam.getDefault();
        final List<String> webcams = new ArrayList<>();
        int camNumber = 1;
        for (final WebcamDevice webcam : Webcam.getDriver().getDevices()) {
            Logger.debug("Webcam (" + camNumber++ + "): " + webcam.getName()); //$NON-NLS-1$ //$NON-NLS-2$
            webcams.add(webcam.getName());
        }
        if (webcams.isEmpty()) {
            webcams.add(WebcamSelector.NO_WEBCAM_FOUND);
        }
        return webcams;
    }

    /**
     * Benachrichtigen der registrierten Listeners.
     */
    private void notifyListeners() {
        for (final WebcamSelectedListener listener : this.listeners) {
            listener.notifyOfWebcamsChanged();
        }
    }
}
