/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.qrscanner.gui.views;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.TabFolder;

import de.frankmuenster.jameica.qrscanner.Settings;
import de.frankmuenster.jameica.qrscanner.gui.controller.SettingsController;
import de.frankmuenster.jameica.qrscanner.utils.I18N;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.ColumnLayout;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.jameica.gui.util.TabGroup;
import de.willuhn.util.ApplicationException;

/**
 * View fr die Einstellungen
 *
 * @author Frank Mnster
 */
public class SettingsView extends AbstractView {

    /**
     *
     */
    private static final String PLACEHOLDER_TEXT = "     "; //$NON-NLS-1$

    /**
     * Unser Kontrolleur
     */
    private final transient SettingsController control;

    /**
     * Der Tabfolder fr die einzelnen Tabs
     */
    private transient TabFolder folder;

    /**
     * Liefert eine neu Settings-View zurck und erstellt einen passenden Controller
     */
    public SettingsView() {

        super();
        this.control = new SettingsController(this);
    }

    /**
     * @see de.willuhn.jameica.gui.AbstractView#bind()
     */
    @Override
    public void bind() throws Exception {

        GUI.getView().setTitle(I18N.tr("_settings")); //$NON-NLS-1$
        final GridLayout grid = new GridLayout(2, false);
        grid.verticalSpacing = 10;
        final TabGroup grundEinstellung = new TabGroup(this.getTabFolder(), I18N.tr("_basicSettings")); //$NON-NLS-1$
        final ColumnLayout colLayout = new ColumnLayout(grundEinstellung.getComposite(), 2);
        final SimpleContainer left = new SimpleContainer(colLayout.getComposite());
        final SimpleContainer right = new SimpleContainer(colLayout.getComposite());
        if (Settings.isHibiscusInstalled()) {
            left.addHeadline(I18N.tr("_defaultSettings")); //$NON-NLS-1$
            left.addLabelPair(I18N.tr("SettingsView.deafultAccount"), this.control.getDefaultAccount()); //$NON-NLS-1$
            left.addLabelPair(I18N.tr("_sepaPurposeCode"), this.control.getPurposeCode()); //$NON-NLS-1$
            left.addText(SettingsView.PLACEHOLDER_TEXT, false);
        }
        left.addHeadline(I18N.tr("SettingsView.ipCamSupport")); //$NON-NLS-1$
        left.addLabelPair(I18N.tr("SettingsView.ipCam"), this.control.getIpCam()); //$NON-NLS-1$
        final ColumnLayout cl1 = new ColumnLayout(left.getComposite(), 2);
        final SimpleContainer host = new SimpleContainer(cl1.getComposite());
        host.addLabelPair(I18N.tr("SettingsView.IpCamHostAndPort"), this.control.getIpCamHost()); //$NON-NLS-1$
        final SimpleContainer port = new SimpleContainer(cl1.getComposite());
        port.addInput(this.control.getIpCamPort());
        left.addText(SettingsView.PLACEHOLDER_TEXT, false);
        left.addHeadline(I18N.tr("SettingsView.supportDataMatrixHeadLine")); //$NON-NLS-1$
        left.addCheckbox(this.control.getDataMatrixHack(), I18N.tr("SettingsView.supportDataMatrix")); //$NON-NLS-1$
        left.addText(I18N.tr("SettingsView.dataMatrixWarning"), //$NON-NLS-1$
                true);
        left.addText(SettingsView.PLACEHOLDER_TEXT, false);
        left.addHeadline(I18N.tr("SettingsView.configuredFoundWebcamsHeadLine")); //$NON-NLS-1$
        left.addPart(this.control.getDiscoverdWebcams());
        left.addText(SettingsView.PLACEHOLDER_TEXT, false);
        left.addPart(this.control.getRefeshBtn());
        right.addText(SettingsView.PLACEHOLDER_TEXT, false);
        final ButtonArea buttons = new ButtonArea();
        buttons.addButton(I18N.tr("_save"), new Action() { //$NON-NLS-1$

            /**
             * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
             */
            @Override
            public void handleAction(final Object context) throws ApplicationException {

                SettingsView.this.control.speichern();
            }
        }, null, true, "document-save.png"); //$NON-NLS-1$
        buttons.paint(this.getParent());
    }

    /**
     * Liefert den Tab-Folder, in dem die einzelnen Module der Einstellungen untergebracht sind.
     *
     * @return der Tab-Folder.
     */
    private TabFolder getTabFolder() {

        if (this.folder == null) {
            this.folder = new TabFolder(this.getParent(), SWT.NONE);
            this.folder.setLayoutData(new GridData(GridData.FILL_BOTH));
        }
        return this.folder;
    }
}
